﻿---
-- Default values for the player saved variables
---
sConfig = {
	general = {
		allowDrag = true,
		allowSettingsClick = true,
		displayLocale = nil,
		showDebug = false,
	},
	messages = {
		playerFloating = true,
		playerChat = false,
		petFloating = true,
		petChat = false,
		bgObjectives = true,
		colors = {
			playerKill = {0.72, 1, 0.71, 1},
			playerQuest = {0.5, 1, 0.7, 1},
			playerBattleground = {1, 0.5, 0.5, 1},
			playerLevel = {0.35, 1, 0.35, 1},
			petKill = {0.52, 0.73, 1, 1},
		},
	},
	averageDisplay = {
		visible = true,
		backdrop = true,
		verbose = true,
		colorText = true,
        header = true,
		tooltip = true,
		playerKills = true,
		playerQuests = true,
		playerBGs = true,
		playerBGOs = false,
		petKills = true,
		playerKillListLength = 10,
		playerQuestListLength = 10,
		playerBGListLength = 30,
		playerBGOListLength = 10,
		petKillListLength = 10,
	},
	ldb = {
		allowTextColor = true,
		showIcon = true,
		showLabel = false,
		showText = true,
		textPattern = "default",
		text = {
			kills = true,
			quests = true,
			bgs = true,
			bgo = false,
			pet = true,
			xp = true,
			petxp = true,
			colorValues = true,
			verbose = true,
		},
		tooltip = {
			showDetails = true,
			showExperience = true,
			showBGInfo = true,
			showPetInfo = true,
		}
	}
}
sData = {
	player = {
		killAverage = 0,
		questAverage = 0,
		killList = {},
		questList = {},
		bgList = {},
	},
	pet = {
		killAverage = 0,
		killList = {},
		xpList = {}
	},
	customPattern = nil,
}

---
-- Verifies that all config values have a default value
-- (New values are sometimes not initialized if older versions of saved values exist)
---
function VerifyConfig()
	-- General
	if sConfig.general == nil then sConfig.general = {  } end
	if sConfig.general.allowDrag == nil then sConfig.general.allowDrag = true end
	if sConfig.general.showDebug == nil then sConfig.general.showDebug = false end
	if sConfig.general.allowSettingsClick == nil then sConfig.general.allowSettingsClick = true end
	if sConfig.general.displayLocale == nil then sConfig.general.displayLocale = GetLocale() end
	
	-- Messages
	if sConfig.messages == nil then sConfig.messages = {  } end
	if sConfig.messages.playerFloating == nil then sConfig.messages.playerFloating = true end
	if sConfig.messages.playerChat == nil then sConfig.messages.playerChat = false end
	if sConfig.messages.petFloating == nil then sConfig.messages.petFloating = true end
	if sConfig.messages.petChat == nil then sConfig.messages.petChat = false end
	if sConfig.messages.bgObjectives == nil then sConfig.messages.bgObjectives = true end
    
    -- Message Colors
	if sConfig.messages.colors == nil then sConfig.messages.colors = {} end
	if sConfig.messages.colors.playerKill == nil then sConfig.messages.colors.playerKill = {0.72, 1, 0.71, nil} end
	if sConfig.messages.colors.playerQuest == nil then sConfig.messages.colors.playerQuest = {0.5, 1, 0.7, nil} end
	if sConfig.messages.colors.playerBattleground == nil then sConfig.messages.colors.playerBattleground = {1, 0.5, 0.5, nil} end
	if sConfig.messages.colors.playerLevel == nil then sConfig.messages.colors.playerLevel = {0.35, 1, 0.35, nil} end
	if sConfig.messages.colors.petKill == nil then sConfig.messages.colors.petKill = {0.52, 0.73, 1, nil} end
    
    if sConfig.messages.colors.playerKill[4] ~= nill then sConfig.messages.colors.playerKill[4] = nil end
    if sConfig.messages.colors.playerQuest[4] ~= nill then sConfig.messages.colors.playerQuest[4] = nil end
    if sConfig.messages.colors.playerKill[4] ~= nill then sConfig.messages.colors.playerBattleground[4] = nil end
    if sConfig.messages.colors.playerLevel[4] ~= nill then sConfig.messages.colors.playerLevel[4] = nil end
    if sConfig.messages.colors.petKill[4] ~= nill then sConfig.messages.colors.petKill[4] = nil end
	
	-- averageDisplay
	if sConfig.averageDisplay == nil then sConfig.averageDisplay = {  } end
	if sConfig.averageDisplay.visible == nil then sConfig.averageDisplay.visible = true end
	if sConfig.averageDisplay.backdrop == nil then sConfig.averageDisplay.backdrop = true end
	if sConfig.averageDisplay.verbose == nil then sConfig.averageDisplay.verbose = true end
	if sConfig.averageDisplay.tooltip == nil then sConfig.averageDisplay.tooltip = true end
	if sConfig.averageDisplay.colorText == nil then sConfig.averageDisplay.colorText = true end
	if sConfig.averageDisplay.header == nil then sConfig.averageDisplay.header = true end
	if sConfig.averageDisplay.playerKills == nil then sConfig.averageDisplay.playerKills = true end
	if sConfig.averageDisplay.playerQuests == nil then sConfig.averageDisplay.playerQuests = true end
	if sConfig.averageDisplay.playerBGs == nil then sConfig.averageDisplay.playerBGs = true end
	if sConfig.averageDisplay.playerBGOs == nil then sConfig.averageDisplay.playerBGOs = false end
	if sConfig.averageDisplay.petKills == nil then sConfig.averageDisplay.petKills = true end
	if sConfig.averageDisplay.playerKillListLength == nil then sConfig.averageDisplay.playerKillListLength = 10 end
	if sConfig.averageDisplay.playerQuestListLength == nil then sConfig.averageDisplay.playerQuestListLength = 10 end
	if sConfig.averageDisplay.playerBGListLength == nil then sConfig.averageDisplay.playerBGListLength = 30 end
	if sConfig.averageDisplay.playerBGOListLength == nil then sConfig.averageDisplay.playerBGOListLength = 30 end
	if sConfig.averageDisplay.petKillListLength == nil then sConfig.averageDisplay.petKillListLength = 10 end

	-- LDB
	if sConfig.ldb == nil then sConfig.ldb = {  } end
	if sConfig.ldb.text == nil then sConfig.ldb.text = {  } end
	if sConfig.ldb.tooltip == nil then sConfig.ldb.tooltip = {  } end
	if sConfig.ldb.allowTextColor == nil then sConfig.ldb.allowTextColor = true end
	if sConfig.ldb.showIcon == nil then sConfig.ldb.showIcon = true end
	if sConfig.ldb.showLabel == nil then sConfig.ldb.showLabel = false end
	if sConfig.ldb.showText == nil then sConfig.ldb.showText = true end
	if sConfig.ldb.textPattern == nil then sConfig.ldb.textPattern = "default" end
	if sConfig.ldb.text.kills == nil then sConfig.ldb.text.kills = true end
	if sConfig.ldb.text.quests == nil then sConfig.ldb.text.quests = true end
	if sConfig.ldb.text.bgs == nil then sConfig.ldb.text.bgs = true end
	if sConfig.ldb.text.bgo == nil then sConfig.ldb.text.bgo = true end
	if sConfig.ldb.text.pet == nil then sConfig.ldb.text.pet = true end
	if sConfig.ldb.text.xp == nil then sConfig.ldb.text.xp = true end
	if sConfig.ldb.text.petxp == nil then sConfig.ldb.text.petxp = true end
	if sConfig.ldb.text.verbose == nil then sConfig.ldb.text.verbose = true end
	if sConfig.ldb.text.colorValues == nil then sConfig.ldb.text.colorValues = true end
	if sConfig.ldb.tooltip.showDetails == nil then sConfig.ldb.tooltip.showDetails = true end
	if sConfig.ldb.tooltip.showExperience == nil then sConfig.ldb.tooltip.showExperience = true end
	if sConfig.ldb.tooltip.showBGInfo == nil then sConfig.ldb.tooltip.showBGInfo = true end
	if sConfig.ldb.tooltip.showPetInfo == nil then sConfig.ldb.tooltip.showPetInfo = true end
	
	-- Data
	if sData.player.killAverage == nil then sData.player.killAverage = 0 end
	if sData.player.questAverage == nil then sData.player.questAverage = 0 end
	if sData.player.killList == nil then sData.player.killList = {} end
	if sData.player.questList == nil then sData.player.questList = {} end
	if sData.player.bgList == nil then sData.player.bgList = {} end
	if sData.pet.killAverage == nil then sData.pet.killAverage = 0 end
	if sData.pet.killList == nil then sData.pet.killList = {} end
	if sData.customPattern == nil then sData.customPattern = 0 end
	
end
