﻿cDisplay = 
{  
    ---
    -- A shortcut to add a message to the default chat window
    printStyle = {
        white = { r=1.0, g=1.0, b=1.0, group=54, addToStart=false },
        gray = { r=0.5, g=0.5, b=0.5, group=53, addToStart=false }
    },
    Print = function(self, message, style, color)
		local r, g, b = unpack(color or {1, 1, 1});
        if style == nil then
            style = self.printStyle.white
        end
        DEFAULT_CHAT_FRAME:AddMessage(message, r, g, b, style.group, style.addToStart);
    end,
	
	Debug = function(self, message)
		if sConfig.general.showDebug then
			self:Print(message, self.printStyle.gray, {0.5, 0.5, 0.5})
		end
	end,

    ---
    -- Average Window controls
    ---
    Average = 
    {
        window = nil,
        lines = {},
        backdrop = nil,
    
        -- Window Style
        visible = true, -- Note that this should be treated as a private member. Use the SetVisibility function.
        showBackground = true,
        textMargin = 5,
        lineSpacing = 2,
        
        ---
        -- Creates the window
        Initialize = function(self)
            if self.window ~= nil then
                return
            end
        
            self.window = XToLevelDisplay
            self.backdrop = self.window:GetBackdrop();
            
            -- Create the lines
            self.lines.header = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_h1')
            self.lines.header:SetText('XToLevel')
            self.lines.header.group = "header"
            self.lines.header.tabIndex = 1
            
            self.lines.playerKills = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerKills:SetText(L["Kills"] .. ': ...')
            self.lines.playerKills.group = "player"
            self.lines.playerKills.tabIndex = 2
            
            self.lines.playerQuests = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerQuests:SetText(L["Quests"] .. ': ...')
            self.lines.playerQuests.group = "player"
            self.lines.playerQuests.tabIndex = 3
			
			self.lines.playerBGs = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerBGs:SetText(L["Battles"] .. ': ...')
            self.lines.playerBGs.group = "player"
            self.lines.playerBGs.tabIndex = 4
			
			self.lines.playerBGOs = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.playerBGOs:SetText(L["Objectives"] .. ': ...')
            self.lines.playerBGOs.group = "player"
            self.lines.playerBGOs.tabIndex = 5
			
			self.lines.petKills = self.window:CreateFontString(nil, 'OVERLAY', 'XToLevel_span')
            self.lines.petKills:SetText(L["Pet"] .. ': ...')
            self.lines.petKills.group = "player"
            self.lines.petKills.tabIndex = 6
            
            self:Update()
            self:updateBackdrop()
        end,
        
        ---
        -- Updates the window
        Update = function(self)
            if self.window == nil then
                return
            end
            
            if not sConfig.averageDisplay.visible then
                self.window:Hide()
                return
            elseif not cPlayer.isActive and not (cPet.isActive or cPet.hasBeenActive) then
				self.window:Hide()
                return
			else
				self.window:Show()
            end
			
			if sConfig.averageDisplay.header then
				self.lines.header:Show()
			end
			
			if not cPlayer.isActive then
				self.lines.playerKills:Hide()
				self.lines.playerQuests:Hide()
			else
				self.lines.playerKills:Show()
				self.lines.playerQuests:Show()
			end
			if not (cPet.isActive or cPet.hasBeenActive) then
				self.lines.petKills:Hide()
			else
				self.lines.petKills:Show()
			end
			if not cPlayer.isActive or (cPlayer:GetAverageBGsRemaining() == nil or not ShowBattlegroundData()) then
				self.lines.playerBGs:Hide()
			else
				self.lines.playerBGs:Show()
			end
			if not cPlayer.isActive or (cPlayer:GetAverageBGObjectivesRemaining() == nil or not ShowBattlegroundData()) then
				self.lines.playerBGOs:Hide()
			else
				self.lines.playerBGOs:Show()
			end
			
            -- Show / Hide the lines and update the anchors
            -- The elements are re-arraged into a temp array according to their tabIndex
            local iLines = {}
            for eName, eElem in pairs(self.lines) do
                iLines[eElem.tabIndex] = { name=eName, elem=eElem }
            end
            
            local nextAnchor = self.window
            local nextMarginV = self.textMargin
            local nextMarginH = self.textMargin
            local nextPoint = "TOPLEFT"
            local currentIndent =  -nextMarginH
            for index, value in ipairs(iLines) do
                local name = value.name
                local elem = value.elem
                
                elem:ClearAllPoints()
                if sConfig.averageDisplay[name] and elem:IsShown() then
                    -- Clear text indent if the previous element was in the same group
                    if nextAnchor.group == elem.group then
                        nextMarginH = 0
                    else
                        currentIndent = currentIndent + nextMarginH
                    end
                    elem:SetPoint("TOPLEFT", nextAnchor, nextPoint, nextMarginH, -nextMarginV)
                    elem.lineIndent = currentIndent
                    
                    nextAnchor = elem
                    nextMarginV = self.lineSpacing
                    nextPoint = "BOTTOMLEFT"
                else
                    elem:Hide()
                end
            end
            
            -- Update messages for the visible elements
            if self.lines.header:IsShown() then
                self.lines.header:SetText('XToLevel')
            end
			
			local labelColor, playerColor, petColor, playerProgress, petProgress;
			labelColor = "ffffff"
			
			if cPlayer.isActive then
				playerProgress = round(cPlayer.currentXP / cPlayer.maxXP * 100, 1)
				if sConfig.averageDisplay.colorText then
					playerProgressColor = GetProgressColor(playerProgress)
				end
				if self.lines.playerKills:IsShown() then
					if sConfig.averageDisplay.colorText then
						self.lines.playerKills:SetText("|cFF".. labelColor .. ((sConfig.averageDisplay.verbose and L["Kills"]) or L["Kills Short"]) .. ':|r |cFF'.. playerProgressColor .. cPlayer:GetAverageKillsRemaining() .."|r")
					else
						self.lines.playerKills:SetText(((sConfig.averageDisplay.verbose and L["Kills"]) or L["Kills Short"]) .. ': ' .. cPlayer:GetAverageKillsRemaining())
					end
				end
				if self.lines.playerQuests:IsShown() then
					if sConfig.averageDisplay.colorText then
						self.lines.playerQuests:SetText("|cFF".. labelColor.. ((sConfig.averageDisplay.verbose and L["Quests"]) or L["Quests Short"]) .. ':|r |cFF'.. playerProgressColor .. cPlayer:GetAverageQuestsRemaining() .."|r")
					else
						self.lines.playerQuests:SetText(((sConfig.averageDisplay.verbose and L["Quests"]) or L["Quests Short"]) .. ': ' .. cPlayer:GetAverageQuestsRemaining())
					end
				end
				if self.lines.playerBGs:IsShown() and cPlayer:GetAverageBGsRemaining() ~= nil then
					if sConfig.averageDisplay.colorText then
						self.lines.playerBGs:SetText("|cFF".. labelColor.. ((sConfig.averageDisplay.verbose and L["Battles"]) or L["Battles Short"]) .. ':|r |cFF'.. playerProgressColor .. cPlayer:GetAverageBGsRemaining() .."|r")
					else
						self.lines.playerBGs:SetText(((sConfig.averageDisplay.verbose and L["Battles"]) or L["Battles Short"]) .. ': ' .. cPlayer:GetAverageBGsRemaining())
					end
				end
				if self.lines.playerBGOs:IsShown() then -- and cPlayer:GetAverageBGObjectivesRemaining() ~= nil
					if sConfig.averageDisplay.colorText then
						self.lines.playerBGOs:SetText("|cFF".. labelColor.. ((sConfig.averageDisplay.verbose and L["Objectives"]) or L["Objectives Short"]) .. ':|r |cFF'.. playerProgressColor .. cPlayer:GetAverageBGObjectivesRemaining() .."|r")
					else
						self.lines.playerBGOs:SetText(((sConfig.averageDisplay.verbose and L["Objectives"]) or L["Objectives Short"]) .. ': ' .. cPlayer:GetAverageBGObjectivesRemaining())
					end
				end
			end
			
			if cPet.isActive and self.lines.petKills:IsShown() then
				petProgress = round(cPet.xp / cPet.maxXP * 100, 1)
				if sConfig.averageDisplay.colorText then
					petProgressColor = GetProgressColor(petProgress)
					self.lines.petKills:SetText("|cFF".. labelColor.. ((sConfig.averageDisplay.verbose and L["Pet"]) or L["Pet Short"]) .. ':|r |cFF'.. petProgressColor .. cPet:GetAverageKillsRemaining() .."|r")
				else
					self.lines.petKills:SetText(((sConfig.averageDisplay.verbose and L["Pet"]) or L["Pet Short"]) .. ': ' .. cPet:GetAverageKillsRemaining())
				end
			end
            
            -- Calculate the height and width of the lines
            local maxWidth = 0
            local totalHeight = self.textMargin * 2
            for name, value in pairs(self.lines) do
                if value:IsVisible() then
                    local currentWidth = value:GetWidth() + (value.lineIndent or 0)
                    if currentWidth  > maxWidth then
                        maxWidth = currentWidth
                    end
                    totalHeight = totalHeight + value:GetHeight() + self.lineSpacing
                end
            end
            local totalWidth = maxWidth + (self.textMargin * 2)
            
            -- Set the display box size
            self.window:SetWidth(totalWidth)
            self.window:SetHeight(totalHeight)
            
            self:updateBackdrop()
        end,
        
        ---
        -- Updates the backdrop
        updateBackdrop = function(self)
            if self.window == nil then
                return
            end
            
            if sConfig.averageDisplay.backdrop then
                self.window:SetBackdrop(self.backdrop);
                self.window:SetBackdropColor("0.0", "0.75", "0.5", "0.75");
                self.window:SetBackdropBorderColor("0.0", "0.0", "0.0", "1.0");
            else
                self.window:SetBackdrop(nil);
            end
        end,
        
        -- Show / Hide the window
        ToggleVisibility = function(self, newValue)
            if self.window == nil then
                return
            end
        
            if(newValue) then
                self.visible = newValue
            else
                self.visible = not self.visible
            end
            
            if self.visible then
                self.window:Show()
            else
                self.window:Hide()
            end
        end
    },
    
    ---
    -- Controls for the floating display
    -- The settings check is added here to simplify the calling code.
    ---
    Floating = 
    {
        killStyle =  { r=0.5, g=1.0, b=0.7, group=56, fade=5 },
		petStyle =  { r=1.0, g=0.7, b=0.5, group=57, fade=5 },
        questStyle = { r=0.5, g=1.0, b=0.7, group=56, fade=5 },
        levelStyle = { r=0.35, g=1.0, b=0.35, group=56, fade=6 },
        
        PrintKill = function(self, mobName, mobsRequired)
            if sConfig.messages.playerFloating then
                local message = mobsRequired .." ".. mobName .. L["Kills Needed"]
                self:Print(message, sConfig.messages.colors.playerKill, self.killStyle)
            end
        end,
		
		PrintPetKill = function(self, petName, mobName, mobsRequired)
            if sConfig.messages.petFloating then
                local message = mobsRequired .. " " .. mobName .. L["Pet Kills Needed"] .. petName
                self:Print(message, sConfig.messages.colors.petKill, self.petStyle)
            end
        end,
        
        PrintQuest = function(self, questsRequired)
            if sConfig.messages.playerFloating then
                local message = questsRequired .. L["Quests Needed"]
                self:Print(message, sConfig.messages.colors.playerQuest, self.questStyle)
            end
        end,
		
		PrintBattleground = function(self, bgsRequired)
            if sConfig.messages.playerFloating then
                local message = bgsRequired .. L["Battlegrounds Needed"]
                self:Print(message, sConfig.messages.colors.playerBattleground, self.questStyle)
            end
        end,
		PrintBGObjective = function(self, bgsRequired)
            if sConfig.messages.playerFloating and sConfig.messages.bgObjectives then
                local message = bgsRequired .. L["Battleground Objectives Needed"]
                self:Print(message, sConfig.messages.colors.playerBattleground, self.questStyle)
            end
        end,
        
        PrintLevel = function(self, level)
            if sConfig.messages.playerFloating then
                local message = L["Level Reached"]
                self:Print(message, sConfig.messages.colors.playerLevel, self.levelStyle)
            end
        end,
        
        Print = function(self, text, color, style)
			local r, g, b = unpack(color)
            UIErrorsFrame:AddMessage(text, r, g, b, style.group, style.fade);
        end
    },
    
    ---
    -- Controls for the chat display
    ---
    Chat = 
    {
        killStyle =  { r=0.5, g=1.0, b=0.7, group=56, addToStart=false },
		petStyle =  { r=1.0, g=0.7, b=0.5, group=57, fade=5, addToStart=false },
        questStyle = { r=0.5, g=1.0, b=0.7, group=56, addToStart=false },
        levelStyle = { r=0.35, g=1.0, b=0.35, group=56, addToStart=false},
        
        PrintKill = function(self, mobName, mobsRequired)
            if sConfig.messages.playerChat then
                local message = mobsRequired .." ".. mobName .. L["Kills Needed"]
                cDisplay:Print(message, self.killStyle, sConfig.messages.colors.playerKill)
            end
        end,
		
		PrintPetKill = function(self, petName, mobName, mobsRequired)
            if sConfig.messages.petChat then
                local message = mobsRequired .. " " .. mobName .. L["Pet Kills Needed"] .. petName
                cDisplay:Print(message, self.petStyle, sConfig.messages.colors.petKill)
            end
        end,
        
        PrintQuest = function(self, questsRequired)
            if sConfig.messages.playerChat then
                local message = questsRequired .. L["Quests Needed"]
                cDisplay:Print(message, self.questStyle, sConfig.messages.colors.playerQuest)
            end
        end,
		
		PrintBattleground = function(self, bgsRequired)
            if sConfig.messages.playerChat then
                local message = bgsRequired .. L["Battlegrounds Needed"]
                cDisplay:Print(message, self.questStyle, sConfig.messages.colors.playerBattleground)
            end
        end,
		PrintBGObjective = function(self, bgsRequired)
            if sConfig.messages.playerChat and sConfig.messages.bgObjectives then
                local message = bgsRequired .. L["Battleground Objectives Needed"]
                cDisplay:Print(message, self.questStyle, sConfig.messages.colors.playerBattleground)
            end
        end,
        
        PrintLevel = function(self, level)
            if sConfig.messages.playerChat then
                local message = L["Level Reached"]
                cDisplay:Print(message, self.levelStyle, sConfig.messages.colors.playerLevel)
            end
        end
    },
	
	---
	-- Tooltip
	---
	Tooltip =
	{
		Show = function(self, frame, a1, f1, a2, footerText)
			if frame ~= nil then
				GameTooltip:SetOwner(frame, "ANCHOR_NONE")
			else
				GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
			end
			if a1 ~= nil or f1 ~= nil or a2 ~= nil then
				GameTooltip:SetPoint(a1, f1, a2)
			end
			
			GameTooltip:ClearLines()
			
			local labelColor, dataColor, footerColor, xpProgress, xpNeededTotal, xpNeededActual;
			local bgs, latestData, averageRaw, averageFormatted, needed;
			
			if sConfig.ldb.allowTextColor then
				labelColor = { r=0.75, g=0.75, b=0.75 }
				dataColor = { r=0.9, g=1, b=0.9 }
				footerColor = { r=0.6, g=0.6, b=0.6 }
			else
				labelColor = {}
				dataColor = {}
				footerColor = {}
			end
			
			GameTooltip:AddLine(L["XToLevel"])
			
			if cPlayer.level < cPlayer.maxLevel then
				if sConfig.ldb.tooltip.showDetails then
					GameTooltip:AddDoubleLine(L["Kills"] .. ":" , NumberFormat(cPlayer:GetAverageKillsRemaining()) .." @ ".. NumberFormat(round(cPlayer:GetAverageKillXP(), 0)) .." xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
					GameTooltip:AddDoubleLine(L["Quests"] .. ":" , NumberFormat(cPlayer:GetAverageQuestsRemaining()) .." @ ".. NumberFormat(round(cPlayer:GetAverageQuestXP(), 0)) .." xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
				end
				if ShowBattlegroundData() then
					GameTooltip:AddDoubleLine(L["Battles"] .. ":" , NumberFormat(cPlayer:GetAverageBGsRemaining() or 0) .." @ ".. NumberFormat(round(cPlayer:GetAverageBGXP(), 0)) .." xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
					GameTooltip:AddDoubleLine(L["Objectives"] .. ":" , NumberFormat(cPlayer:GetAverageBGObjectivesRemaining() or 0) .." @ ".. NumberFormat(round(cPlayer:GetAverageBGObjectiveXP(), 0)) .." xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
				end
				GameTooltip:AddLine(" ")
				
				if sConfig.ldb.tooltip.showExperience then
					xpProgress = round((cPlayer.currentXP or 0) / (cPlayer.maxXP or 1) * 100, 1)
					xpNeededTotal = cPlayer.maxXP - cPlayer.currentXP
					xpNeededActual = cPlayer:GetKillsRequired(1) or "~"
					
					GameTooltip:AddLine(L["Experience"] .. ": ")
					GameTooltip:AddDoubleLine(" " .. L["XP Progress"] .. ": " , ShrinkNumber(UnitXP("player")) .. " / " .. ShrinkNumber(UnitXPMax("player")) .. " [" .. tostring(xpProgress) .. "%" .. "]", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
					GameTooltip:AddDoubleLine(" " .. L["XP Rested"] .. ": " , NumberFormat(cPlayer:IsRested() or 0) .. " xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
					GameTooltip:AddDoubleLine(" " .. L["Quest XP Required"] .. ": " , NumberFormat(xpNeededTotal) .. " xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
					GameTooltip:AddDoubleLine(" " .. L["Kill XP Required"] .. ": " , NumberFormat(xpNeededActual) .. " xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
					
					GameTooltip:AddLine(" ")
				end
			end
			
			if (cPet.isActive or cPet.hasBeenActive) and sConfig.ldb.tooltip.showPetInfo then
				xpProgress = cPet.xp / cPet.maxXP * 100
				GameTooltip:AddLine(L["Pet"] .. ":")
				GameTooltip:AddDoubleLine(" " .. L["XP Progress"] .. ":" , NumberFormat(round((cPet.xp or 0))).." / "..NumberFormat(round((cPet.maxXP or 0))).." ["..NumberFormat(round(xpProgress, 1)).."%]", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
				GameTooltip:AddDoubleLine(" " .. L["Kill XP Required"] .. ":" , NumberFormat(round((cPet.maxXP or 0) - (cPet.xp or 0))) .. " xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
				GameTooltip:AddDoubleLine(" " .. L["Kills"] .. ":" , NumberFormat(cPet:GetAverageKillsRemaining()).." @ "..NumberFormat(round(cPet:GetAverageKillXP(), 0)).." xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)		
				GameTooltip:AddLine(" ")
			end
				
			if ShowBattlegroundData() then
				if (# sData.player.bgList) > 0 then
					bgs = cPlayer:GetBattlegroundsListed()
					latestData = cPlayer:GetLatestBattlegroundDetails();
					
					if bgs ~= nil then
						GameTooltip:AddLine(L['Battlegrounds Required'] .. ":")
							for name, count in pairs(bgs) do
								if name == false then
									name = "Unknown"
								end
								averageRaw = cPlayer:GetBattlegroundAverage(name)
								if averageRaw == 0 then
									averageRaw = latestData.totalXP
								end
								averageFormatted = NumberFormat(round(averageRaw, 0))
								needed = cPlayer:GetQuestsRequired(tonumber(averageRaw))
								GameTooltip:AddDoubleLine(" ".. name .. ": " , needed .. " @ ".. averageFormatted .. " xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
							end
						GameTooltip:AddLine(" ")
					end
					
					
					
					if sData.player.bgList[1].inProgress then
						GameTooltip:AddLine(L['Current Battleground'] .. ":")
					else
						GameTooltip:AddLine(L['Last Battleground'] .. ":")
						if sData.player.bgList[1].name ~= false then
							GameTooltip:AddDoubleLine(" ".. L['Name'] ..": " , sData.player.bgList[1].name, labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
						end
					end
					
					GameTooltip:AddDoubleLine(" ".. L['Total XP'] ..": " , NumberFormat(latestData.totalXP), labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
					GameTooltip:AddDoubleLine(" ".. L['Objectives'] ..": " , NumberFormat(latestData.objCount) .." @ ".. NumberFormat(latestData.xpPerObj) .." xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
					GameTooltip:AddDoubleLine(" ".. L['NPC Kills'] ..": " , NumberFormat(latestData.killCount) .." @ ".. NumberFormat(latestData.xpPerKill) .." xp", labelColor.r, labelColor.g, labelColor.b, dataColor.r, dataColor.b, dataColor.b)
					GameTooltip:AddLine(" ")
				else
					GameTooltip:AddLine(L['Battlegrounds Required'] .. ":")
					GameTooltip:AddLine(" " .. L['No Battles Fought'], labelColor.r, labelColor.g, labelColor.b)
					GameTooltip:AddLine(" ")
				end
			end

			if footerText ~= nil then
				GameTooltip:AddLine(tostring(footerText), footerColor.r, footerColor.g, footerColor.b)
			end
			
			labelColor = nil;
			dataColor = nil;
			footerColor = nil;
			xpProgress = nil;
			xpNeededTotal = nil;
			xpNeededActual = nil;
			bgs = nil;
			latestData = nil;
			averageRaw = nil;
			averageFormatted = nil;
			needed = nil;
			
			GameTooltip:Show()
		end,
		
		Hide = function(self)
			GameTooltip:Hide();
		end
	},
    
	---
	-- LDB
	---
	LDB = 
	{
        -- Constants
        textPatterns = {
			default = "{kills}{$seperator: }{color=cfcfdf}{$label}:{/color} {progress}{$value}{/progress}{/kills}{quests}{$seperator: }{color=cfcfdf}{$label}:{/color} {progress}{$value}{/progress}{/quests}{bgs}{$seperator: }{color=cfcfdf}{$label}:{/color} {progress}{$value}{/progress}{/bgs}{bgo}{$seperator: }{color=cfcfdf}{$label}:{/color} {progress}{$value}{/progress}{/bgo}{xp}{$seperator: }{progress}[{$value}]{/progress}{/xp}{pet}{$seperator: }{color=cfcfdf}{$label}:{/color} {progress}{$value}{/progress}{/pet}{petxp}{$seperator: }{progress}[{$value}]{/progress}{/petxp}",
			minimal = "{kills}{progress}{$value}{/progress}{/kills}{quests}{color=cfcfdf}{$seperator:/}{/color}{progress}{$value}{/progress}{/quests}{bgs}{color=cfcfdf}{$seperator:/}{/color}{progress}{$value}{/progress}{/bgs}{xp}{color=cfcfdf}{$seperator:/}{/color}{progress}{$value}{/progress}{/xp}{pet}{color=cfcfdf}{$seperator:/}{/color}{progress}{$value}{/progress}{/pet}{petxp}{color=cfcfdf}{$seperator:/}{/color}{progress}{$value}{/progress}{/petxp}",
			minimal_dashed = "{kills}{progress}{$value}{/progress}{/kills}{quests}{color=cfcfdf}{$seperator:-}{/color}{progress}{$value}{/progress}{/quests}{bgs}{color=cfcfdf}{$seperator:-}{/color}{progress}{$value}{/progress}{/bgs}{xp}{color=cfcfdf}{$seperator:-}{/color}{progress}{$value}{/progress}{/xp}{pet}{color=cfcfdf}{$seperator:-}{/color}{progress}{$value}{/progress}{/pet}{petxp}{color=cfcfdf}{$seperator:-}{/color}{progress}{$value}{/progress}{/petxp}",
			brackets = "{kills}{progress}[{$value}]{/progress}{/kills}{quests}{progress}[{$value}]{/progress}{/quests}{bgs}{progress}[{$value}]{/progress}{/bgs}{xp}{progress}[{$value}]{/progress}{/xp}{pet}{$seperator:-}{progress}[{$value}]{/progress}{/pet}{petxp}{progress}[{$value}]{/progress}{/petxp}",
		},
		textTags = {
			[1] = { tag = "kills", label = 'init', value = '~', color = nil, },
			[2] = { tag = "quests", label = 'init', value = '~', color = nil, }, 
			[3] = { tag = "bgs", label = 'init', value = '~', color = nil, }, 
			[4] = { tag = "bgo", label = 'init', value = '~', color = nil, }, 
			[5] = { tag = "xp", label = 'init', value = '~', color = nil, },
			[6] = { tag = "pet", label = 'init', value = '~', color = nil, },
			[7] = { tag = "petxp", label = 'init', value = '~', color = nil, },
		},
    
		-- Members
		dataObject = nil,
		mouseOver = false,
        currentPattern = nil,
		
		-- Constructor
		Initialize = function(self)
			-- Initialize the data object
			local iconName = (UnitFactionGroup("player") == "Alliance") and "INV_Jewelry_TrinketPVP_01" or "INV_Jewelry_TrinketPVP_02"
			local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
			self.dataObject = ldb:NewDataObject("XToLevel", {
				type = "data source",
				icon = "Interface\\Icons\\" .. iconName,
				text = "XToLevel",
				label = "XToLevel",
                version = "3.2.2_8b1",
                align = "right",
			});
			ldb = nil;
            
			-- Set data object events
            function self.dataObject:OnEnter()
                cDisplay.LDB.mouseOver = true;
                local a1, f1, a2 = FindAnchor(self);
                cDisplay.Tooltip:Show(self, a1, f1, a2, L['Click To Configure']);
			end
            function self.dataObject:OnLeave()
				cDisplay.LDB.mouseOver = false;
				cDisplay.Tooltip:Hide()
			end
			function self.dataObject:OnClick(button)
				XToLevelConfig:Show();
				XToLevelConfig_SwapPanels('LDB Text');
			end
			
            self:BuildPattern();-- /run cDisplay.LDB:BuildPattern(); cDisplay.LDB:Update();
			self:Update();
		end,
        
        -- Creates the text pattern to be used.
        BuildPattern = function(self)
            if ((cPlayer.level or 1) < (cPlayer.maxLevel or 1)) or cPet.isActive or cPet.hasBeenActive then
                local showPlayer, showPet, playerProgress, playerProgressColor, petProgress, petProgressColor;
                local newText, useColors, isFirst, sPos, ePost, value, rest, attributes, out;
            
                showPlayer = (cPlayer.level < cPlayer.maxLevel)
                showPet = cPet.isActive or cPet.hasBeenActive
                
                -- User is leveling, proceed normally.
                if showPlayer then
                    playerProgress = round(cPlayer.currentXP / cPlayer.maxXP * 100, 1)
                    playerProgressColor = GetProgressColor(playerProgress)
                end
                    
                if showPet then
                    petProgress = round(cPet.xp / cPet.maxXP * 100, 1)
                    petProgressColor = GetProgressColor(petProgress)
                end
                
                -- Load the appropriate pattern, attempting to find one in the config.
                newText = self.textPatterns.default
                if sConfig.ldb.textPattern ~= nil then
                    if sConfig.ldb.textPattern == "custom" then
                        newText = sData.customPattern or "Please enter a pattern."
                        
                        -- Parse html-like syntax
                        newText = string.gsub(newText, '<(%w+) ?(.-)>', function(tag, attr)
                            attributes = {
                                label = false,
                                post = false,
                                seperator = false,
                            }
                            
                            for key, value in pairs(attributes) do
                                sPos, ePost, value, rest = string.find(attr, key ..'="([^"]-)"')
                                if value ~= nil then
                                    attributes[key] = value
                                end
                            end
                            
                            out = "{".. tag .."}"
                            if attributes.seperator then
                                out = out .. "{$seperator:" .. attributes.seperator .."}"
                            end
                            if attributes.label then
                                out = out .. attributes.label ..": "
                            end
                            out = out .. "{progress}{$value}"
                            if attributes.post then
                                out = out .. attributes.post or nil
                            end
                            out = out .. "{/progress}{/" .. tag .."}"
                            return out
                        end)
                    else
                        newText = self.textPatterns[sConfig.ldb.textPattern]
                    end
                end
                
                -- Replace {color} tags
                if sConfig.ldb.allowTextColor then 
                    newText = string.gsub(newText, "{color=([0-9A-Fa-f]+)}(.-){/color}", "|cFF%1%2|r")
                else
                    newText = string.gsub(newText, "{color=[0-9A-Fa-f]+}(.-){/color}", "%1")
                end
                
                -- Prepare tags
                useColors = (sConfig.ldb.allowTextColor and sConfig.ldb.text.colorValues)
                self.textTags = {
                    [1] = {
                        tag = "kills",
                        label = (sConfig.ldb.text.verbose and L["Kills"] ) or L["Kills Short"],
                        value = (showPlayer and '$$kills$$') or nil,
                        color = (useColors and '$$playercolor$$') or nil,
                    },
                    [2] = { 
                        tag = "quests",
                        label = (sConfig.ldb.text.verbose and L["Quests"] ) or L["Quests Short"],
                        value = (showPlayer and '$$quests$$') or nil,
                        color = (useColors and '$$playercolor$$') or nil,
                    }, 
                    [3] = { 
                        tag = "bgs",
                        label = (sConfig.ldb.text.verbose and L["Battles"] ) or L["Battles Short"],
                        value = ((showPlayer and UnitLevel("Player") >= 10) and '$$bgs$$') or nil,
                        color = (useColors and '$$playercolor$$') or nil,
                    }, 
                    [4] = { 
                        tag = "bgo",
                        label = (sConfig.ldb.text.verbose and L["Objectives"] ) or L["Objectives Short"],
                        value = ((showPlayer and UnitLevel("Player") >= 10) and '$$bgo$$') or nil,
                        color = (useColors and '$$playercolor$$') or nil,
                    }, 
                    [5] = { 
                        tag = "xp",
                        label = L["XP"],
                        value = (showPlayer and ('$$xp$$' .. "%%")) or nil,
                        color = (sConfig.ldb.allowTextColor and '$$playercolor$$') or nil,
                    },
                    [6] = { 
                        tag = "pet",
                        label = (sConfig.ldb.text.verbose and L["Pet"] ) or L["Pet Short"],
                        value = (showPet and '$$pet$$') or nil,
                        color = (useColors and '$$petcolor$$') or nil,
                    },
                    [7] = { 
                        tag = "petxp",
                        label = L["XP"],
                        value = (showPet and '$$petxp$$'  .. "%%") or nil,
                        color = (sConfig.ldb.allowTextColor and '$$petcolor$$') or nil,
                    },
                }
                
                -- Replace values
                isFirst = true
                for i, object in ipairs(self.textTags) do
                    if sConfig.ldb.text[object.tag] and object.value ~= nil then
                        newText = string.gsub(newText, "{".. object.tag .."}(.-){/".. object.tag .."}", function(str)
                            str = string.gsub(str, "{$label}", object.label)
                            str = string.gsub(str, "{$value}", object.value)
                            if object.color ~= nil then
                                str = string.gsub(str, "{progress}(.-){/progress}", "|cFF".. object.color .."%1|r")
                            else
                                str = string.gsub(str, "{progress}(.-){/progress}", "%1")
                            end
                            if isFirst then
                                str = string.gsub(str, "{$seperator(.-)}", "")
                            else
                                str = string.gsub(str, "{$seperator:?(.-)}", "%1")
                            end
                            return str
                        end)
                        isFirst = false
                    else
                        newText = string.gsub(newText, "{".. object.tag .."}.-{/".. object.tag .."}", "")
                    end
                end
                self.currentPattern = newText
                
                -- Free resources
                showPlayer = nil;
                showPet = nil;
                playerProgress = nil;
                playerProgressColor = nil;
                petProgress = nil;
                petProgressColor = nil;
                newText = nil;
                useColors = nil;
                isFirst = nil;
                sPos = nil;
                ePost = nil;
                value = nil;
                rest = nil;
                attributes = nil;
                out = nil;
                
            else
                -- Player is at max level and has no pets.
                if sConfig.ldb.customColors then
                    self.currentPattern = "|cFFaaaaaaInactive|r"
                else
                    self.currentPattern = "Inactive"
                end
            end
        end,
		
		-- Update LDB text
		-- Note, this is a very memory demanding function.
		--  Calling it periodically is not a good idea!
		Update = function(self)
			if self.dataObject == nil then
				return false;
			end
		
			if sConfig.ldb.showLabel then
				self.dataObject.label = L["XToLevel"]
			else
				self.dataObject.label = nil
			end
			
			if sConfig.ldb.showIcon then
				local iconName = (UnitFactionGroup("player") == "Alliance") and "INV_Jewelry_TrinketPVP_01" or "INV_Jewelry_TrinketPVP_02"
				self.dataObject.icon = "Interface\\Icons\\" .. iconName
			else
				self.dataObject.icon = nil
			end
			
			if sConfig.ldb.showText then
                local pattern = self.currentPattern;
                if (cPlayer.level < cPlayer.maxLevel) then
                    local playerProgress, playerProgressColor;
                    playerProgress = round(cPlayer.currentXP / cPlayer.maxXP * 100, 1)
                    playerProgressColor = GetProgressColor(playerProgress)
                    
                    pattern = string.gsub(pattern, '%$%$playercolor%$%$', playerProgressColor);
                    pattern = string.gsub(pattern, '%$%$kills%$%$', (round(cPlayer:GetAverageKillsRemaining()) or "~"));
                    pattern = string.gsub(pattern, '%$%$quests%$%$', (round(cPlayer:GetAverageQuestsRemaining()) or "~"));
                    pattern = string.gsub(pattern, '%$%$xp%$%$', playerProgress or "~");
                    pattern = string.gsub(pattern, '%$%$bgs%$%$', (round(cPlayer:GetAverageBGsRemaining()) or "~"));
                    pattern = string.gsub(pattern, '%$%$bgo%$%$', (round(cPlayer:GetAverageBGObjectivesRemaining()) or "~"));
                else
                    pattern = string.gsub(pattern, '%$%$playercolor%$%$', '');
                    pattern = string.gsub(pattern, '%$%$kills%$%$', '');
                    pattern = string.gsub(pattern, '%$%$quests%$%$', '');
                    pattern = string.gsub(pattern, '%$%$xp%$%$', '');
                    pattern = string.gsub(pattern, '%$%$bgs%$%$', '');
                    pattern = string.gsub(pattern, '%$%$bgo%$%$', '');
                end
                    
                if cPet.isActive or cPet.hasBeenActive then
                    local petProgress, petProgressColor;
                    petProgress = round(cPet.xp / cPet.maxXP * 100, 1)
                    petProgressColor = GetProgressColor(petProgress)
                    
                    pattern = string.gsub(pattern, '%$%$petcolor%$%$', petProgressColor);
                    pattern = string.gsub(pattern, '%$%$pet%$%$', (round(cPet:GetAverageKillsRemaining()) or "~"));
                    pattern = string.gsub(pattern, '%$%$petxp%$%$', petProgress or "~");
                else
                    pattern = string.gsub(pattern, '%$%$petcolor%$%$', '');
                    pattern = string.gsub(pattern, '%$%$pet%$%$', '');
                    pattern = string.gsub(pattern, '%$%$petxp%$%$', '');
                end
   
                
				self.dataObject.text = pattern;
			else
				self.dataObject.text = nil
			end
		end,
	},
}
